package supperSlonic.dynamoDBDAL;

import java.util.List;

import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.dynamodbv2.model.AttributeDefinition;
import com.amazonaws.services.dynamodbv2.model.DeleteTableRequest;
import com.amazonaws.services.dynamodbv2.model.CreateTableRequest;
import com.amazonaws.services.dynamodbv2.model.DescribeTableRequest;
import com.amazonaws.services.dynamodbv2.model.KeySchemaElement;
import com.amazonaws.services.dynamodbv2.model.ResourceNotFoundException;
import com.amazonaws.services.dynamodbv2.model.TableDescription;
import com.amazonaws.services.dynamodbv2.model.TableStatus;

import supperSlonic.amazonConnection.AmazonContext;
import supperSlonic.dynamoDBDAL.Table.DynamoDBSchema;

public class DeployDAL extends BaseDAL<DynamoDBSchema>
{	
	public DeployDAL(AmazonContext context, DynamoDBSchema tableSchema)
	{
		super(context, tableSchema);
	}
	
	public boolean tableExists()
    {
    	DescribeTableRequest describeTableRequest = new DescribeTableRequest();
    	describeTableRequest.withTableName(this.dynamoDBTable.getTableName());
    	
    	try
    	{
    		this.amazonDynamoDBClient.describeTable(describeTableRequest);
    		return true;
    	}
    	catch(ResourceNotFoundException ex)
    	{
    		return false;
    	}
    }
	
	public void deleteTable()
	{
		DeleteTableRequest deleteTableRequest = new DeleteTableRequest(this.dynamoDBTable.getTableName());
		this.amazonDynamoDBClient.deleteTable(deleteTableRequest);
		System.out.println("Table deleted");
	}
    
    public void createTable()
    {
        List<KeySchemaElement> keySchema = this.dynamoDBTable.createKeySchema();
        List<AttributeDefinition> attributeDefinitions = this.dynamoDBTable.createAttributeDefinitions();

        // Create a table with a primary key named 'name', which holds a string
        CreateTableRequest createTableRequest = new CreateTableRequest()
        	.withTableName(this.dynamoDBTable.getTableName())
        	.withAttributeDefinitions(attributeDefinitions)
        	.withKeySchema(keySchema)
            .withProvisionedThroughput(this.dynamoDBTable.createProvisionedThroughput());
        
        TableDescription createdTableDescription = amazonDynamoDBClient.createTable(createTableRequest).getTableDescription();
        System.out.println("Created Table: " + createdTableDescription);

        // Wait for it to become active
        waitForTableToBecomeAvailable(this.dynamoDBTable.getTableName());
    }
    
    protected void waitForTableToBecomeAvailable(String tableName)
    {
        System.out.println("Waiting for " + tableName + " to become ACTIVE...");

        long startTime = System.currentTimeMillis();
        long endTime = startTime + (10 * 60 * 1000);
        while (System.currentTimeMillis() < endTime)
        {
            try
            {
                Thread.sleep(1000 * 20);
            }
            catch (Exception e) {}
            
            
                DescribeTableRequest request = new DescribeTableRequest().withTableName(tableName);
                TableDescription tableDescription = amazonDynamoDBClient.describeTable(request).getTable();
                String tableStatus = tableDescription.getTableStatus();
                System.out.println("  - current state: " + tableStatus);
                if (tableStatus.equals(TableStatus.ACTIVE.toString()))
                    return;
        }

        throw new AmazonServiceException("Table " + tableName + " never went active");
    }
}
